/* TEST PROGRAM 3 FOR 8515 Development Kit
   
   This program shows how to:
   
   1 acquire 12 bit samples from ADC MCP3208 input 1
     (set speed variable: ACQ_RATE !) and store them on E2PROM 
     (24LC512 address 001)
     (leds on PORTC are used to monitor the memory usage)
     
   2 transfer the stored data to a PC via the serial port RS232
     (use "hyperterminal" to show data)
     
   
   hardware involved:microprocessor, ADC, EEPROM, leds, piezo, RS232.
   
*/

#include <avr\io.h>
#include <stdio.h>
#include <avr\interrupt.h>
#include <avr\signal.h>
#include <avr\pgmspace.h>
#include <ctype.h>

#include "uart.h"
#include "i2cmaster.h"


#define XTAL_CPU         8000000      /* 8Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */


#define E2P_1  0xA2	// device address of EEPROM 24LCXX on slot1
			// EEPROM slot2 -->0xA4
			// RTC slot 0 -->0xA0

#define ACQ_RATE  1	// adc sampling rate 540= 1 sample each 2.7s (24h acquisition)



void delay(unsigned int p);
void beep(unsigned int t);
void readadc(unsigned char channel,unsigned char mode);
void statusbar(unsigned int ledsc);

void write_e2p(unsigned char device_addr,unsigned int loc_addr,\
unsigned char datamsb,unsigned char datalsb);

unsigned char read_e2p(unsigned char device_addr,unsigned int loc_addr);



unsigned char adc_lsb;
unsigned char adc_msb;
unsigned int adc_data;
unsigned char ret;
unsigned char e2pdata;
unsigned char counter=0;
unsigned char c;
unsigned char sbuf[5];


int main(void)
{
  unsigned int eeaddr=0;//16bits
  
  outp(0xff,DDRC);	//PORT C as output
  outp(0xff,PORTC);	//all pins to 1  (means all leds off)

  outp(0x80,DDRD);	//PORT D input except PD7 output(for buzzer)
  outp(0x7f,PORTD);	//pull-up , PD7 to 0
  			//uart rx/tx pins are automatically configured
	
  outp(0xB0,DDRB);	//PB0-3 IN , PB4-PB7 OUT except MISO (PB6)
  outp(0xFF,PORTB);	//outputs to 1, inputs with pull-up
  
  i2c_init();		// init I2C interface (on PB0,1)
  
  outp(0x5C,SPCR);	//SPI Control Register settings:
			//SPIE=0 interrupt disabled
			//SPE=1 SPI enabled 
			//DORD=0 MSB trasmitted first
			//MSTR=1 master mode
			//CPOL=1 clk 1 when idle
			//CPHA=1 clock phase (acquisition pos.edge)
			//SPR1,SPR0=0,0 fclock/4 (2MHz)
  			
  beep(4000);		//1 second beep 4000
	
  delay(200);		//1 second delay 200*5ms

if (!(PIND & 0x04))	// check if PD2 input is 0 (jumper to GND connected)
	{		// if PD2=0 data recording routine is executed
	
	eeaddr=1;		//leave free eeprom address 0
	
 	while (eeaddr<0xFA00)	// LOOP until e2prom memory on slot1 is full
		{
		readadc(1,1);	//read CH1 single ended
	
		write_e2p(E2P_1,eeaddr,adc_msb,adc_lsb);//write adc 12bit data on eeprom 1
	
		eeaddr=eeaddr+2;	//increment address on eeprom 1
	
		statusbar(eeaddr);	// leds on PORTC show percentage of used memory
		
		delay(ACQ_RATE);		// delay each sample
		
	
		}
	}//endif
	 
	
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,XTAL_CPU) );// STARTING RS232 COMMUNICATION

  sei();						//enable interrups for uart
  
while(1)			//loop forever
	{	
  	eeaddr=1;
  	beep(1000);
  
  	uart_puts("Download data? (Press Y to start)");	//download data from UART to PC
  	uart_putc('\r');
  	uart_putc('\n');
  
  	c = uart_getc();	
	while (c!='Y') c = uart_getc();			// wait until Y is pressed
 	
	while (eeaddr<=0xFA00)
		{
	
		e2pdata = read_e2p(E2P_1,eeaddr);	//read eeprom location
	
		eeaddr++;				//incr. loc. address
			
		if (counter){				//newline each 16bit data
		
			adc_data=adc_data|e2pdata;	//2nd LSB byte
			
			itoa(adc_data,sbuf,10);		//convert int to ascii
			
			if (adc_data<10) uart_putc(sbuf[0]);
			else if (adc_data<100){
				uart_putc(sbuf[0]);
				uart_putc(sbuf[1]);
				}
			else if (adc_data<1000){
				uart_putc(sbuf[0]);
				uart_putc(sbuf[1]);
				uart_putc(sbuf[2]);
				}
			else uart_puts(sbuf);

			uart_putc('\r');		//newline
			uart_putc('\n');
			counter=0;
			}
		else {
			counter++;
			adc_data = e2pdata;			//1st MSB byte
			adc_data = (adc_data<<8) & 0xFF00;
			}
		}
	}
		

}	//END OF MAIN


	
unsigned char read_e2p(unsigned char device_addr,unsigned int loc_addr)
{
  unsigned char data;

  i2c_start_wait(device_addr+I2C_WRITE);
  
  i2c_write(loc_addr>>8);
  i2c_write(loc_addr);			//set 16 bit location address
  
  i2c_rep_start(device_addr+I2C_READ);
  
  data = i2c_readNak();			//read 1 bytes nak
  i2c_stop();
  
  return (data);					
}
	

void write_e2p(unsigned char device_addr,unsigned int loc_addr,\
unsigned char datamsb,unsigned char datalsb)
{	
  i2c_start_wait(device_addr+I2C_WRITE);	//address i2c eeprom 
	
  i2c_write(loc_addr>>8);
  i2c_write(loc_addr);	//set 16 bit location address
  i2c_write(datamsb);	//write 2 bytes in "page write" mode
  i2c_write(datalsb);
  i2c_stop();
		
}

void statusbar(unsigned int ledsc)
{ 
  unsigned int led;
  
  led=0;
  
  led=(ledsc/8000);   		//  ledsc/(0xfa00/8)
  
  PORTC=PORTC&(~BV(led));
}
  
  

void readadc(unsigned char channel,unsigned char mode)
{
  unsigned char dataw1=0;
  unsigned char dataw2=0;
  unsigned char dataw3=0;
  
  if (channel&0x04) dataw1=(dataw1|0x01);	//set dataw1
  if (mode==1) dataw1=(dataw1|0x02);
  dataw1=(dataw1|0x04);
  
  if (channel&0x02) dataw2=(dataw2|0x80);	//set dataw2
  if (channel&0x01) dataw2=(dataw2|0x40);
  
  cbi(PORTB,4);				//enable slave (SS pin)			
  outp(dataw1,SPDR);			//sends dataw1
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
 
  outp(dataw2,SPDR);			//sends dataw2
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
  adc_msb=inp(SPDR)&0x0F;		//read msb data(ignore 1st 4 bits)
  
  outp(dataw3,SPDR);			//sends dataw3(=0)
  while(bit_is_clear(SPSR,SPIF));	//wait until bit SPIF is set
  sbi(PORTB,4);				//dis. slave (end)
  adc_lsb=inp(SPDR);			//read lsb data word

}

void delay(unsigned int p)   // total delay= p x 5 ms
{
  unsigned int i;
  unsigned char j;
  unsigned char k;
  for(i=0;i<p;i++){			//delay =40000*125ns=5ms
  
    for (j=0;j<100;j++)	{		// (395+1)*100=39600+(4*100)=40000
    	asm("nop");			// 1 cycle
	for (k=0;k<79;k++) asm("nop");	//(4+1)cycles *79= 395
	 }    
	}
}



void beep(unsigned int t)	//buzzer on PD7 ; duration=t*250us (4000=1s)
{
  unsigned int i;
  unsigned char j;
  unsigned char k;

for (i=0;i<t;i++){			//126us*2= 252us period (4KHz)

  sbi(PORTD,7);
    for (j=0;j<2;j++)			//(500+4)*2=1008 *125ns=126us
      for (k=0;k<100;k++) asm("nop");	//(4+1)cycles*100=500
      
  cbi(PORTD,7);
    for (j=0;j<2;j++)			//(500+4)*2=1008 *125ns=126us
      for (k=0;k<100;k++) asm("nop");	//(4+1)cycles*100=500
  }
cbi(PORTD,7);
}






